﻿using System.Text;

namespace MinimalisticUIFramework {
	#region Part 1 (panels support) solution

	public abstract class Panel : Control {
		protected List<Control> _children { get; } = new();

		public override string ToString() {
			var sb = new StringBuilder();
			sb.AppendLine($"{this.GetType().Name} {{");
			for (int i = 0; i < _children.Count; i++) {
				sb.AppendLine($"{_children[i]} {GetChildDetailsString(i)}");
			}
			sb.Append("}");
			return sb.ToString();
		}

		protected virtual string GetChildDetailsString(int childIndex) => "";
	}

	//

	public class StackPanel : Panel {
		public void AddChild(Control control) { _children.Add(control); }
	}

	//

	public struct Point {
		public int X { get; set; }
		public int Y { get; set; }
	}

	public class Canvas : Panel {
		private List<Point> _childLocations = new();

		public void AddChild(Control control, Point p) {
			_children.Add(control);
			_childLocations.Add(p);
		}

		protected override string GetChildDetailsString(int childIndex) => $"at {_childLocations[childIndex].X}, {_childLocations[childIndex].Y}";
	}

	#endregion
	#region Part 2 (fluent syntax) solution

	public static class ControlExtensionsForStackPanel {
		public static T PlacedIn<T>(this T control, StackPanel stackPanel) where T : Control {
			stackPanel.AddChild(control);
			return control;
		}
	}

	//

	public static class ControlExtensionsForCanvas {
		public static UnresolvedCanvasPlacement<T> PlacedIn<T>(this T control, Canvas canvas) where T : Control {
			return new UnresolvedCanvasPlacement<T>(canvas, control);
		}
	}

	public struct UnresolvedCanvasPlacement<TOriginalControl> where TOriginalControl : Control {
		private Canvas _targetCanvas;
		private TOriginalControl _placedControl;

		public UnresolvedCanvasPlacement(Canvas targetCanvas, TOriginalControl placedControl) {
			_targetCanvas = targetCanvas;
			_placedControl = placedControl;
		}

		public TOriginalControl At(int x, int y) {
			_targetCanvas.AddChild(_placedControl, new Point { X = x, Y = y });
			return _placedControl;
		}
	}

	#endregion
}
